package pfg.gca;

import java.util.Date;

/**
 * A GC record represents a single garbage collection activity.
 * It is independent of whether the GC occured due to a manual
 * GC request or an allocation failure.
 * 
 * @author Paul Glezen
 */
public class GCRecord
{
   /** See {@link #getGcIndex}.       */
   protected int gcIndex;
   
   /** See {@link #getMarkTime}.      */
   protected int mark;
   
   /** See {@link #getSweepTime}.     */
   protected int sweep;
   
   /** See {@link #getCompactTime}.   */
   protected int compact;
   
   /** See {@link #getStartTime}.     */
   protected Date startTime;
   
   /** See {@link #getBytesReclaimed}.*/
   protected long bytesReclaimed;
   
   /** See {@link #getBytesFree}.     */
   protected long bytesFree;
   
   /** See {@link #getBytesAvailable}.*/
   protected long bytesAvailable;
   
   /**  Create an empty struct.*/
   public GCRecord()
   {
      startTime = new Date();
   }

   /**
    * Create a new GCRecord.
    * <p>
    * @param gcIndex        the integer associated with this GC activity
    * @param mark           mark time
    * @param sweep          sweep time
    * @param compact        compaction time
    * @param startTime      time at which this GC activity began
    * @param bytesReclaimed bytes reclaimed by this GC activity
    * @param bytesFree      bytes available on the heap after GC
    * @param bytesAvailable total bytes available on heap
    */   
   public GCRecord(int gcIndex,
                   int mark,
                   int sweep,
                   int compact,
                   Date startTime,
                   long bytesReclaimed,
                   long bytesFree,
                   long bytesAvailable)
   {
   	this.gcIndex        = gcIndex;
   	this.mark           = mark;
   	this.sweep          = sweep;
   	this.compact        = compact;
   	this.startTime      = startTime;
   	this.bytesReclaimed = bytesReclaimed;
   	this.bytesFree      = bytesFree;
   	this.bytesAvailable = bytesAvailable;
   }


   /**
    * Get bytes presently available to the heap.  This includes
    * memory that is being used as well as memory available for
    * allocation of new objects.
    * <p>
    * @return memory available in bytes
    */
   public long getBytesAvailable()
   {
      return bytesAvailable;
   }

   /**
    * Get the amounts of bytes free after a GC.  The bytes free
    * refers to heap bytes available for allocation of new objects.
    * <p>
    * @return memory available (in bytes) for new object allocation
    */
   public long getBytesFree()
   {
      return bytesFree;
   }

   /**
    * Get bytes freed by the GC.
    * <p>
    * @return memory reclaimed in bytes
    */
   public long getBytesReclaimed()
   {
      return bytesReclaimed;
   }

   /**
    * Get the compaction time.  In the mark and sweep strategy,
    * compaction is only performed on an as-needed basis.  When
    * compaction is not needed, this time will be zero.
    * <p>
    * @return compaction time in milliseconds
    */
   public int getCompactTime()
   {
      return compact;
   }

   /**
    * Get the index for this GC activity.  Each GC activity
    * has an associated index.
    */
   public int getGcIndex()
   {
      return gcIndex;
   }

   /**
    * Set the bytes available on the heap.
    * <p>
    * @param bytesAvailable
    */
   public void setBytesAvailable(long bytesAvailable)
   {
      this.bytesAvailable = bytesAvailable;
   }

   /**
    * Set the number of bytes freed by this GC activity.
    * <p>
    * @param bytesFree
    */
   public void setBytesFree(long bytesFree)
   {
      this.bytesFree = bytesFree;
   }

   /**
    * Set the bytes reclaimed by this GC activity.
    * <p>
    * @param bytesReclaimed 
    */
   public void setBytesReclaimed(long bytesReclaimed)
   {
      this.bytesReclaimed = bytesReclaimed;
   }

   /**
    * Set the GC compaction time.
    * <p>
    * @param compact compaction time in milliseconds
    */
   public void setCompactTime(int compact)
   {
      this.compact = compact;
   }

   /**
    * Get the index for this GC activity.  Each GC is assigned an
    * index by the JVM.
    * <p>
    * @param gcIndex the index for this GC activity
    */
   public void setGcIndex(int gcIndex)
   {
      this.gcIndex = gcIndex;
   }

   /**
    * Get the start time for this GC activity.
    * <p>
    * @return time GC activity started as <code>Date</code> object
    */
   public Date getStartTime()
   {
      return startTime;
   }

   /**
    * Set the start time of this GC activity.
    * <p>
    * @param startTime start time as a <code>Date</code> object
    */
   public void setStartTime(Date startTime)
   {
      this.startTime = startTime;
   }

   /**
    * Get the mark time.  The mark time is the first stage of the
    * "mark and sweep" strategy in which it is determined which 
    * memory blocks that are no longer referenced.  These are 
    * marked for reclaimation.
    * <p>
    * @return mark time in milliseconds
    */
   public int getMarkTime()
   {
      return mark;
   }

   /**
    * Get the sweep time.  The sweep time is the second stage of the
    * "mark and sweep" strategy in which memory blocks that have been
    * marked for reclaimation are freed.
    * <p>
    * @return sweep time in milliseconds
    */
   public int getSweepTime()
   {
      return sweep;
   }

   /**
    * Set the mark time.
    * <p>
    * @param mark mark time in milliseconds
    */
   public void setMarkTime(int mark)
   {
      this.mark = mark;
   }

   /**
    * Set the sweep time.
    * <p>
    * @param sweep sweep time in milliseconds
    */
   public void setSweepTime(int sweep)
   {
      this.sweep = sweep;
   }

   /**
    * Return the total time for this GC activity.  The total time is
    * the sum of the {@link #getMarkTime mark time}, the {@link
    * #getSweepTime sweep time} and the {@link #getCompactTime
    * compaction time}.
    */
   public int getTotalTime()
   {
   	return (mark + sweep + compact);
   }

   /**
    * Send a one-line summary of the contents of this GC activity
    * to system-out.
    */
   public String toString()
   {
      StringBuffer result = new StringBuffer(200);
      result.append("GC(" + gcIndex + ") " + startTime + " (Reclaimed/free/total) = ");
      result.append(" (" + bytesReclaimed + "/" + bytesFree + "/" + bytesAvailable + ")");
      result.append(" (mark/sweep/compact) = (" + mark + "/" + sweep + "/" + compact + ")");
      return result.toString();
   }

}
