package pfg.gca;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Properties;

/**
 * Manage properties in a property file rather than in memory.
 * This class's intent is to persist user perferences.  The
 * preferences are read from storage when this class is instantiated
 * and written back to storage with each property set.
 * <p>
 * The filename is determined by {@link #BaseFilename}.  The
 * directory is determined by the <tt>user.home</tt> system
 * property.
 */
public class PersistProps {
   
   /** Base filename for property file - currently "<b>gca.properties</b>". */
   public static final String BaseFilename = "gca.properties";

   private String filename = null;
   private Properties props = new Properties();

   /**
    * Read properties from a file if they exist.
    * The filename is determined by {@link #BaseFilename}.  The
    * directory is determined by the <tt>user.home</tt> system
    * property.
    */
   public PersistProps() {
      filename = System.getProperty("user.home") + "/" + BaseFilename;
      try {
         InputStream userPropFile = new FileInputStream(filename);
         props.load(userPropFile);
      } catch (FileNotFoundException fnfe) {
         // No preferences found.  That's fine; we'll start from scratch.
      } catch (IOException ioe) {
         System.err.println("Trouble loading user properties from " + filename);
      }
   }
   
   public String getProperty(String propName) {
      return props.getProperty(propName);
   }
   
   public Enumeration propertyNames() {
      return props.propertyNames();
   }
   
   /**
    * Each set invokes the persistence mechanism.
    * <p>
    * @param name name of property to persist
    * @param value value of property to persist
    */
   public void setProperty(String name, String value) {
      props.setProperty(name, value);
		try {
			OutputStream outputStream = new FileOutputStream(filename);
			props.store(outputStream, "GCA Properties");
		} catch (FileNotFoundException e) {
         System.err.println("PersistProps: " + filename + " not found.");
         System.err.println("Changes in user preferences will not be saved.");
		} catch (IOException e) {
         System.err.println("PersistProps: Error in writing properties to " + filename);
         System.err.println("Changes in user preferences will not be saved.");
		}
   }
}
