package pfg.gca.collector.regexp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.regexp.RE;
import org.apache.regexp.RECompiler;
import org.apache.regexp.REProgram;
import org.apache.regexp.RESyntaxException;

import pfg.gca.AFRecord;
import pfg.gca.GCRecord;
import pfg.gca.collector.ICollector;

/**
 * A GC record collector for IBM JDKs: 1.3.1, 1.4.1 and
 * 1.4.2.  This class was originally targeted specifically
 * to the IBM 1.3.1 JDK, hence the name.  Since the next
 * two JDKs did not change the verbose GC record format,
 * this class is used for those as well.
 * <p>
 * The suffix "a" in this class name has no significance.  It
 * was merely made in anticipation that there might be other
 * implementations that could then use "b", "c" and so on.
 * <p>
 * @author Paul Glezen
 */
public class IBM131a implements ICollector
{
	private RE gcHeader;
	private RE gcFreed;
	private RE gcTimes;
	private RE afHeader;
	private RE afAction;
	private RE afComplete;
   
   //----------------------------------------------------------------
   //
   //  Define regular expressions used to parse raw verbose gc output.
   
   //  This line indicates the beginning of a new GC record.
   //
   // <GC(34): GC cycle started Sun Aug 11 00:08:50 2002
   //     --                        --- -- -- -- -- ----
   //     $1                        $2  $3 $4 $5 $6  $7
   public static final String RE_GC_HEADER = "<GC\\((\\d+)\\): GC cycle started .{3} (\\w{3}) (\\d+) (\\d+):(\\d+):(\\d+) (\\d+)";

   //  This line specifies how much space was freed and how long the
   //  overall GC took.  It also tells provides the heap left after GC.
   //
   // <GC(34): freed 5949384 bytes, 33% free (6558112/19790336), in 101 ms>
   //     --         -------        --        ------- --------      ---
   //     $1           $2           $3          $4       $5          $6
   public static final String RE_GC_FREED  = "<GC\\((\\d+)\\): freed (\\d+) bytes, (\\d+)% free \\((\\d+)/(\\d+)\\), in (\\d+) ms>";

   //  This line provides details of the mark/sweep/compact timings.
   //  It is the last line of the GC record used by this program.
   //
   //   <GC(19): mark: 37 ms, sweep: 4 ms, compact: 150 ms>
   //       --         --            -              ---
   //       $1         $2            $3             $4
   public static final String RE_GC_TIMES  = "<GC\\((\\d+)\\): mark: (\\d+) ms, sweep: (\\d+) ms, compact: (\\d+) ms>";
	
	private static Map months = new HashMap(13);
	
   //  Initialize a hash storing the months as specified in
   //  the verbosegc of IBM JDK 1.3.1.
   //
	static 
	{
		months.put("Jan", new Integer(0));
		months.put("Feb", new Integer(1));
		months.put("Mar", new Integer(2));
		months.put("Apr", new Integer(3));
		months.put("May", new Integer(4));
		months.put("Jun", new Integer(5));
		months.put("Jul", new Integer(6));
		months.put("Aug", new Integer(7));
		months.put("Sep", new Integer(8));
      months.put("Oct", new Integer(9));
		months.put("Nov", new Integer(10));
		months.put("Dec", new Integer(11));
	};
	

   /**
    * Compile the required regular expressions.
    */
   public IBM131a()
   {
      try
      {
      	REProgram  regexProgram  = null;
      	RECompiler regexCompiler = new RECompiler();
      	
      	regexProgram = regexCompiler.compile(RE_GC_HEADER);
         gcHeader     = new RE(regexProgram);
         
         regexProgram = regexCompiler.compile(RE_GC_FREED);
         gcFreed      = new RE(regexProgram);

         regexProgram = regexCompiler.compile(RE_GC_TIMES);
         gcTimes      = new RE(regexProgram);
      }
      catch (RESyntaxException e)
      {
      	System.out.println("Regexp parsing exception: " + e);
      }
   }
   	
   
   /**
    * Read raw verbose GC text from a <code>Reader</code> to
    * create a <code>List</code> of {@link pfg.gca.GCRecord}
    * instances.  This method is where most of the collector
    * work is done.
    */
   public List getGCList(Reader reader) throws IOException
   {
   	String line = null;
   	int currentIndex = 0;
   	int thisIndex = 0;
   	GCRecord gcRec = null;
   	AFRecord afRec = null;
   	List result = new LinkedList();
   	BufferedReader lines = new BufferedReader(reader);
   	while ((line = lines.readLine()) != null)
   	{
   		if (gcHeader.match(line))
   		{
   			if (gcRec == null)
   			{
   			   gcRec = new GCRecord();
   			   result.add(gcRec);
   			}
            try
            {
               thisIndex = Integer.parseInt(gcHeader.getParen(1));
               gcRec.setGcIndex(thisIndex);
               // Get the date.
               int      year = Integer.parseInt(gcHeader.getParen(7));
               Integer month = (Integer) months.get(gcHeader.getParen(2));
               int       day = Integer.parseInt(gcHeader.getParen(3));
               int      hour = Integer.parseInt(gcHeader.getParen(4));
               int    minute = Integer.parseInt(gcHeader.getParen(5));
               int    second = Integer.parseInt(gcHeader.getParen(6));
               GregorianCalendar calendar = new GregorianCalendar(year, month.intValue(), day, hour, minute, second);
               gcRec.setStartTime(calendar.getTime());
            }
            catch (NumberFormatException e)
            {
            	System.out.println("Trouble formatting GC Header" + e);
            }
   		}
   		
   		if (gcFreed .match(line))
   		{
   			try
   			{
   				gcRec.setBytesReclaimed(Long.parseLong(gcFreed.getParen(2)));
   				gcRec.setBytesFree(     Long.parseLong(gcFreed.getParen(4)));
   				gcRec.setBytesAvailable(Long.parseLong(gcFreed.getParen(5)));
   			}
            catch (NumberFormatException e)
            {
            	System.out.println("Trouble formatting GC Freed " + e);
            }
   		}
   		
   		if (gcTimes .match(line))
   		{
   			try
   			{
   				gcRec.setMarkTime(Integer.parseInt(gcTimes.getParen(2)));
   				gcRec.setSweepTime(Integer.parseInt(gcTimes.getParen(3)));
   				gcRec.setCompactTime(Integer.parseInt(gcTimes.getParen(4)));
   			}
            catch (NumberFormatException e)
            {
            	System.out.println("Trouble formatting GC Times " + e);
            }
            //  Since this is the last GC line processed, set the GC reference
            //  to null.  There may still be a afRec reference to the record.
            //
            gcRec = null;
   		}
   	}
   	lines.close();
   	return result;
   }
   
   /**
    * Return a description of the JDKs supported by this collector.
    * In this case, these JDKs are
    * <p><ol>
    * <li>IBM JDK 1.3.1
    * <li>IBM JDK 1.4.1
    * <li>IBM JDK 1.4.2
    * </ol><p>
    */
	public String getDescription() {
		return "ibm131 - IBM JDK 1.3.1, 1.4.1 and 1.4.2";
	}

}
