package pfg.gca.collector.regexp;

import java.util.HashMap;

import pfg.gca.collector.CollectorFactory;
import pfg.gca.collector.ICollector;

/**
 * This factory determines a collector among regular
 * expression collectors based on the JDK version.
 * <p>
 * @author Paul Glezen
 */
public class RegExpCollectorFactory extends CollectorFactory
{
   private HashMap  collectors    = null;
 
   /**
    * Populates a <code>HashMap</code> with collector instances.
    * The keys constitute the valid JDK versions.  Presently,
    * these are
    * <p><ul>
    * <li><tt>ibm130</tt>
    * <li><tt>ibm131</tt>
    * </ul><p>
    * Since the IBM 1.4.1 and 1.4.2 JDKs have the same verbose GC
    * format as the IBM 1.3.1 JDK, the <tt>ibm131a</tt> collector
    * is used for all three.
    */  
   public RegExpCollectorFactory()
   {
      super();
      collectors = new HashMap(4);
      collectors.put("ibm130", new IBM130a());
      collectors.put("ibm131", new IBM131a());
   }

   /**
    * Retrieve a collector from a hashtable based on the
    * jdk string.  The input argument is automatically
    * lower-cased to avoid case-sensitivity problems.
    * See {@link #RegExpCollectorFactory} for valid
    * values.
    * <p>
    * The same collector instance is returned on each
    * call for a given jdk type.  If you need more than
    * one instance of the same collector type, use mulitple
    * collector factories.
    * <p>
    * @param jdk case insensitive jdk string
    */
   public ICollector getCollector(String jdk)
   {
      ICollector collector = null;
      collector = (ICollector)collectors.get(jdk);
      return collector;
   }
   
   /**
    * Return an array of supported JDK strings.
    */
   public String[] getSupportedJDKs()
   {
      String[] result = (String[]) collectors.keySet().toArray(new String[0]);
      return result;
   }

   /**
    * Return a description of which JDKs are supported by a collector.
    * <p>
    * @param a tag identifying the collector
    * @return a description of supported JDKs for this collector
    */
	public String getCollectorDescription(String jdkString) {
      String result = null;
      ICollector collector = null;
      if (jdkString != null) {
         collector = (ICollector)collectors.get(jdkString);
         if (collector != null) {
            result = collector.getDescription();
         } else {
            System.err.println("RegExpCollectionFactory: invalid JDK string: " + jdkString);
            System.err.println("\tNo description found.");
         }
      } else {
         System.err.println("RegExpCollectionFactory: null JDK string.  No description found");
      }
		return result;
	}
}
