package pfg.gca.swing;

import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;

import pfg.gca.Context;
import pfg.gca.models.HistogramModel;
import pfg.gca.views.IntegerHistogram;
import pfg.gca.views.TextSummary;

/**
 * This class represents the top-level window for the
 * Swing version of the GCA GUI.
 * <p>
 * @author Paul Glezen
 */
public class GCAFrame extends JFrame
{
   private static final String TITLE = "GC Analyzer";   
   
   private Context gcaContext = null;
   private JComponent    view = null;
   private String     version = null;
   private String       title = null;

   public GCAFrame(Context context)
   {
      super(TITLE);
      addWindowClosedListener();
      gcaContext = context;
      version = context.getProperty(Context.PROP_VERSION);
      title   = context.getProperty(Context.PROP_TITLE);
      
      if (version == null)
      {
         version = "x.x.x";
      }
      if (title != null)
      {
         setTitle(title);
      }
      
      addMenus();
      setSize(450, 450);
      setVisible(true);
   }

   private void addWindowClosedListener()
   {
      addWindowListener( new WindowAdapter()
   	{
   	   public void windowClosing(WindowEvent e)
   		{
   			Window w = e.getWindow();
   			w.setVisible(false);
   			w.dispose();
   			System.exit(0);
   		}
      });
   }
   
   /**
    * Add menu items and define the corresponding actions.
    */
   private void addMenus()
   {
      JMenu fileMenu = new JMenu("File");
      JMenu viewMenu = new JMenu("Views");
   
      //  "File" menu actions.
      //
      fileMenu.add(new AbstractAction("Open ...")
      {
         public void actionPerformed(ActionEvent evt)
         {
            TraceChooser tc = new TraceChooser(GCAFrame.this, gcaContext);
            tc.show();
            setTitle("GCA: " + gcaContext.getCurrentTraceFilename());
         }
      });
      fileMenu.addSeparator();      
      fileMenu.add(new AbstractAction("Exit") 
      {
         public void actionPerformed(ActionEvent evt)
         {
            GCAFrame.this.setVisible(false);
            GCAFrame.this.dispose();
            System.exit(0);
         }
      });
      
      //  View menu actions.
      //
      viewMenu.add(new AbstractAction ("Text Summary") 
      {
         public void actionPerformed(ActionEvent evt)
         {
            if (gcaContext.getGCList() == null)
            {
               String noTraceMsg = "No trace data available.\n" +
                                   "Choose File -> Open to load a trace.";
               JOptionPane.showMessageDialog(GCAFrame.this, noTraceMsg,
                    "Warning", JOptionPane.WARNING_MESSAGE);
            }
            else
            {
               TextSummary ts = new TextSummary(gcaContext.getGCList());
               ts.printSummary();
            }
         }
      });
      viewMenu.add(new AbstractAction("Histogram")
      {
         public void actionPerformed(ActionEvent evt)
         {
            if (view != null)
            {
               view.setVisible(false);
            }
            
            if (gcaContext.getGCList() == null)
            {
               String noTraceMsg = "No trace data available.\n" +
                                   "Choose File -> Open to load a trace.";
               JOptionPane.showMessageDialog(GCAFrame.this, noTraceMsg,
                    "Warning", JOptionPane.WARNING_MESSAGE);
            }
            else
            {
               HistogramModel model = new HistogramModel(gcaContext.getGCList());
               view = new IntegerHistogram(model);
               getContentPane().add(view);
               show();
            }
         }
      });
      
      JMenu helpMenu = new JMenu("Help");
      helpMenu.add(new AbstractAction("Docs")
      {
         public void actionPerformed(ActionEvent evt)
         {
            String docMsg = "Online documentation is available at\nhttp://glezen.org/gca/index.html";
            JOptionPane.showMessageDialog(GCAFrame.this, docMsg);
         }
      });
      helpMenu.addSeparator();
      helpMenu.add(new AbstractAction("About")
      {
         public void actionPerformed(ActionEvent evt)
         {
            String aboutMsg = title + " " + version + "\n" +
                              "Paul Glezen\n" + 
                              "pglezen@us.ibm.com";
            JOptionPane.showMessageDialog(GCAFrame.this, aboutMsg);
         }
      });
            
      
      JMenuBar menuBar = new JMenuBar();
      menuBar.add(fileMenu);
      menuBar.add(viewMenu);
      menuBar.add(helpMenu);
      setJMenuBar(menuBar);
   }
}
