package pfg.gca.views;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JComponent;

import pfg.gca.GCRecord;
import pfg.gca.models.HistogramModel;

/**
 * Display a histogram with integer values.
 */
public class IntegerHistogram extends JComponent
{
	protected static final int XTIC_HEIGHT         = 25;
	protected static final int XLABEL_HEIGHT       = 15;
	protected static final int YTIC_WIDTH          = 25;
	protected static final int YLABEL_WIDTH        = 25;
   protected static final int READABLE_TIC_WIDTH  = 30;
   protected static final int READABLE_TIC_HEIGHT = 20;
	
	protected Map data         = null;
	protected int maxValue     = 0;
	protected int maxFrequency = 0;
	
	protected int    graphHeight  = 0;
	protected int    graphWidth   = 0;
	protected int    xticHeight   = XTIC_HEIGHT;
	protected double xticWidth    = 0d;
	protected double yticHeight   = 0d;
	protected int    yticWidth    = YTIC_WIDTH;
	protected int    xlabelHeight = XLABEL_HEIGHT;
	protected int    xlabelWidth  = 0;
	protected int    ylabelHeight = 0;
	protected int    ylabelWidth  = YLABEL_WIDTH;

   
   /**
    * The <tt>Map</tt> is expected to possess <tt>Integer</tt> keys
    * with <tt>Integer</tt> values.  The maximum values and maximum
    * frequencies are calculated here.
    */
   public IntegerHistogram(Map histogramData)
   {
   	data = histogramData;
   	
   	//  Establish max value and max frequency
   	//
   	if (data != null)
   	{
   		Iterator it = data.keySet().iterator();
   		Integer value = null;
   		Integer frequency = null;
   		while (it.hasNext())
   		{
   			value = (Integer) it.next();
   			frequency = (Integer) data.get(value);
   			if (value.intValue() > maxValue)
   			   maxValue = value.intValue();
   			if (frequency != null  &&  frequency.intValue() > maxFrequency)
   				maxFrequency = frequency.intValue();
   		}
   	}
   }
   
   /**
    * Invoked when the window is first painted and each time it is resized
    * or gets focus.
    */
   protected void paintComponent(Graphics g)
   {
      Graphics2D g2 = (Graphics2D) g;
      g2.setPaint(Color.red);
      
      graphWidth  = getWidth()  - ylabelWidth  - yticWidth;
      graphHeight = getHeight() - xlabelHeight - xticHeight;
      
      xlabelWidth  = graphWidth;
      ylabelHeight = graphHeight;

      xticWidth = graphWidth  / (double) (maxValue + 1);
      yticHeight = graphHeight / (double) (maxFrequency + 1);

//      System.out.println("(width/height) = (" + getWidth() + "/" + getHeight() + ")");
//      System.out.println("(maxValue/maxFrequency) = (" + maxValue + "/" + maxFrequency + ")");
//      System.out.println("(xticWidth/yticHeight) = (" + xticWidth + "/" + yticHeight + ")");
      
      drawGraph(g2);
      drawXAxis(g2);
      drawYAxis(g2);
   }

   /**
    * Draws histogram bars.
    */
   protected void drawGraph(Graphics2D g2)
   {
      //  Determine the transform to scale the graphics appropriately
      //
      Line2D.Double line = null;
      Rectangle2D.Double rectangle = null;
      Integer     value = null;
      Integer frequency = null;
      double x, y, width, height;
      Iterator iter = data.keySet().iterator();
      while (iter.hasNext())
      {
      	value = (Integer) iter.next();
      	frequency = (Integer) data.get(value);
         x = yticWidth + ylabelWidth + value.doubleValue() * xticWidth;
         y = graphHeight - frequency.doubleValue() * yticHeight;
         width  = xticWidth;
         height = frequency.doubleValue() * yticHeight;
         rectangle = new Rectangle2D.Double(x, y, width, height);
         g2.fill(rectangle);
      }
   }
   
   /**
    * Draws x-axis, x tic marks and tic mark labels.
    */
   protected void drawXAxis(Graphics2D g2) 
   {
   	//  Save current color and set our own color.
   	//
   	Color previousColor = g2.getColor();
   	g2.setColor(Color.black);
   	
   	//  Draw Axis Label
   	//
   	String axisTitle = "Garbage Collection Times (msec)";
   	FontMetrics fm   = g2.getFontMetrics();
   	int titleWidth   = fm.stringWidth(axisTitle);
   	int xtitleOffset  = 0;
   	if (titleWidth < graphWidth)
   		xtitleOffset = (graphWidth - titleWidth) / 2;
   	g2.drawString(axisTitle, yticWidth + ylabelWidth + xtitleOffset,
   	                         graphHeight + xticHeight + xlabelHeight - 5);
   	                         
   	double x1, y1, x2, y2;
   	Line2D.Double line = null;
   	
   	//  Draw Axis
   	//
   	x1 = ylabelWidth + yticWidth;
   	x2 = x1 + graphWidth;
   	y1 = y2 = graphHeight;
   	line = new Line2D.Double(x1, y1, x2, y2);
   	g2.draw(line);
   	
   	//  Tic marks and labels
   	//
   	int ticSkip = (int)(READABLE_TIC_WIDTH / xticWidth);
      if (ticSkip == 0)
         ticSkip = 1;
   	int ticCount = 0;
   	int y3 = 0;
   	int xlabel, ylabel;
   	String value = null;
   	int fontHeight = fm.getHeight();
   	int fontWidth = 0;
   	y1 = graphHeight;
   	y2 = graphHeight - 5;  //  Upper tic.
   	y3 = graphHeight + 3;  //  Lower tic.
   	x2 = x1 = yticWidth + ylabelWidth;
   	while (x1 + xticWidth < yticWidth + ylabelWidth + graphWidth)
   	{
   		x1 += xticWidth;
   		line = new Line2D.Double(x1, y1, x1, y2);
   		g2.draw(line);
   		if (ticCount % ticSkip  ==  0)
   		{
   		   x2 = (x1 + x2) / 2;
   		   y3 = graphHeight + 3;
   		   line = new Line2D.Double(x2, y1, x2, y3);
   		   g2.draw(line);
   	      value = String.valueOf(ticCount);
   	      fontWidth = fm.stringWidth(value);
   	      ylabel = graphHeight + 3 + fontHeight;
   	      xlabel = (int) x2 - fontWidth / 2;
   	      g2.drawString(value, xlabel, ylabel);
   		}
   		x2 = x1;
   		ticCount++;
   	}
   	
   	//  Restore color settings.
   	//
   	g2.setColor(previousColor);
   }
   
   /**
    * Draws y-axis, y tic marks and y tic mark labels.
    */
   protected void drawYAxis(Graphics2D g2) 
   {
   	//  Save current color and set our own color.
   	//
   	Color previousColor = g2.getColor();
   	g2.setColor(Color.black);
   	
   	//  Draw Label
   	//
   	String axisTitle = "Number of Occurences";
   	FontMetrics fm   = g2.getFontMetrics();
   	int titleWidth   = fm.stringWidth(axisTitle);
   	int ytitleOffset  = graphHeight;
   	if (titleWidth < graphHeight)
   	   ytitleOffset = (titleWidth + graphHeight) / 2;
   	g2.rotate(-3.141592654d / 2, ylabelWidth, ytitleOffset);
   	g2.drawString(axisTitle, ylabelWidth, ytitleOffset);
   	g2.rotate( 3.141592654d / 2, ylabelWidth, ytitleOffset);
   	
   	//  Draw Axis
   	//
   	double x1, y1, x2, y2;
   	x1 = x2 = ylabelWidth + yticWidth;
   	y1 = graphHeight;
   	y2 = 0;
   	Line2D.Double line = new Line2D.Double(x1, y1, x2, y2);
   	g2.draw(line);   	   	
   	
      //  Tic marks and labels
      //
      int ticSkip = (int)(READABLE_TIC_HEIGHT / yticHeight);
      if (ticSkip == 0)
         ticSkip = 1;
      int ticCount = 1;
      int xlabel, ylabel;
      String value = null;
      double x3 = 0;
      int fontHeight = fm.getHeight();
      int fontWidth = 0;
      x1 = yticWidth + ylabelWidth;
      x2 = x1 + 5;  //  Right tic.
      x3 = x1 - 3;  //  Left tic.
      y1 = graphHeight;
      while (y1 - yticHeight > 0)
      {
         y1 -= yticHeight;
         line = new Line2D.Double(x1, y1, x2, y1);
         g2.draw(line);
         if (ticCount % ticSkip  ==  0)
         {
            line = new Line2D.Double(x1, y1, x3, y1);
            g2.draw(line);
            value = String.valueOf(ticCount);
            fontWidth = fm.stringWidth(value);
            ylabel = (int) y1 + fontHeight / 2;
            xlabel = (int) x3 - fontWidth - 2;
            g2.drawString(value, xlabel, ylabel);
         }
         ticCount++;
      }
      
   	//  Restore color settings.
   	//
   	g2.setColor(previousColor);
   }
   
}
